#include <SDL.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/uio.h>
#include <unistd.h>
#include "sdlrecord.h"

/* sdldump
 *
 * This program will output the contents of a previously recorded SDL session
 *
 * Columns are: raw timestamp, time since previous, event
 */

char *default_filename=DEFAULT_FILE;

int main(int argc,char **argv) {
    char *fn;
    int fd;
    const struct timeval tp;
    struct timeval last;
    int diff;
    sdlr_header header;
    SDL_Event e;

    if(argc>1) { /* provided on command line */
	fn=argv[1];
    } else {
	if((fn=getenv("SDL_RECORD_PATH")) == NULL ) {
	    fn=default_filename;
	}
    } 

    if((fd=open(fn,O_RDONLY))<0) {
	fprintf(stderr,"Can't open filename %s: %s\n",fn,strerror(errno));
	exit(1);
    }
    if(read(fd,&header,sizeof(header))!=sizeof(header)) {
	fprintf(stderr,"Error reading header\n");
	exit(1);
    }
    if(header.version!=FILEVERSION) {
	fprintf(stderr,"Wrong file version %d\n",header.version);
	exit(1);
    }
    printf("%d.%06d %6d Begin\n",(int)header.starttime.tv_sec,(int)header.starttime.tv_usec,0);

    last=header.starttime;

    while((read(fd,(void *)&tp,sizeof(tp))==sizeof(tp))&&(read(fd,&e,sizeof(SDL_Event))==sizeof(SDL_Event))) {

	diff=(tp.tv_sec-last.tv_sec) * 1000 + (tp.tv_usec-last.tv_usec)/1000;
	last=tp;
	printf("%d.%06d %6d ",(int)tp.tv_sec,(int)tp.tv_usec,diff);
	switch(e.type) {
	    case SDL_ACTIVEEVENT:
	    	printf("SDL_ACTIVEEVENT: gain=%d state=%d\n",(int)e.active.gain,(int)e.active.state);
		break;
	    case SDL_KEYDOWN:
	    	printf("SDL_KEYDOWN: state=%d keysym=%d mod=%d\n",(int)e.key.state,(int)e.key.keysym.sym,(int)e.key.keysym.mod);
		break;
	    case SDL_KEYUP:
	    	printf("SDL_KEYUP: state=%d keysym=%d mod=%d\n",(int)e.key.state,(int)e.key.keysym.sym,(int)e.key.keysym.mod);
		break;
	    case SDL_MOUSEMOTION:
	    	printf("SDL_MOUSEMOTION: state=%d x=%d y=%d xrel=%d yrel=%d\n",(int)e.motion.state,(int)e.motion.x,(int)e.motion.y,(int)e.motion.xrel,(int)e.motion.yrel);
		break;
	    case SDL_MOUSEBUTTONDOWN:
	    	printf("SDL_BUTTONDOWN: button=%d state=%d x=%d y=%d\n",(int)e.button.button,(int)e.button.state,(int)e.button.x,(int)e.button.y);
		break;
	    case SDL_MOUSEBUTTONUP:
	    	printf("SDL_BUTTONUP: button=%d state=%d x=%d y=%d\n",(int)e.button.button,(int)e.button.state,(int)e.button.x,(int)e.button.y);
		break;
	    case SDL_JOYAXISMOTION:
	    	printf("SDL_JOYAXISMOTION: which=%d axis=%d value=%d\n",(int)e.jaxis.which,(int)e.jaxis.axis,(int)e.jaxis.value);
		break;
	    case SDL_JOYBALLMOTION:
	    	printf("SDL_JOYBALLMOTION: which=%d ball=%d xrel=%d yrel=%d\n",(int)e.jball.which,(int)e.jball.ball,(int)e.jball.xrel,(int)e.jball.yrel);
		break;
	    case SDL_JOYHATMOTION:
	    	printf("SDL_JOYHATMOTION: which=%d hat=%d value=%d\n",(int)e.jhat.which,(int)e.jhat.hat,(int)e.jhat.value);
		break;
	    case SDL_JOYBUTTONDOWN:
	    	printf("SDL_JOYBUTTONDOWN: which=%d button=%d state=%d\n",(int)e.jbutton.which,(int)e.jbutton.button,(int)e.jbutton.state);
		break;
	    case SDL_JOYBUTTONUP:
	    	printf("SDL_JOYBUTTONUP: which=%d button=%d state=%d\n",(int)e.jbutton.which,(int)e.jbutton.button,(int)e.jbutton.state);
		break;
	    case SDL_VIDEORESIZE:
	    	printf("SDL_VIDEORESIZE: w=%d h=%d\n",(int)e.resize.w,(int)e.resize.h);
		break;
	    case SDL_VIDEOEXPOSE:
	    	printf("SDL_VIDEOEXPOSE\n");
		break;
	    case SDL_QUIT:
	    	printf("SDL_QUIT\n");
		break;
	    case SDL_USEREVENT:
	    	printf("SDL_USEREVENT: code=%d\n",e.user.code);
		break;
	    case SDL_SYSWMEVENT:
	    	printf("SDL_SYSWMEVENT\n");
		break;
	    default:
	    	printf("*Unknown*\n");
		break;
	}
    }
    close(fd);
    return(0);
}

