
#include "SDL/SDL.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/uio.h>
#include <unistd.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <errno.h>
#include <sys/time.h>
#include <time.h>

#include "sdlrecord.h"

int call_sdl_func(int (*f)(),SDL_Event *e);
int SDL_PollEvent(SDL_Event *e);
int SDL_WaitEvent(SDL_Event *e);


char *default_filename=DEFAULT_FILE;

int SDL_PollEvent(SDL_Event *e) {
    static int (*realpe)()=NULL;
    
    if(!realpe) {
	realpe=get_dlfn("SDL_PollEvent");
    }
    return(call_sdl_func(realpe,e));
}

int SDL_WaitEvent(SDL_Event *e) {
    static int (*realwe)()=NULL;
   
    if(!realwe) {
	realwe=get_dlfn("SDL_WaitEvent");
    }
    return(call_sdl_func(realwe,e));
}

int open_file(void) {
    char *fn;
    int out;
    char err_buff[128];
    sdlr_header header;
    
    if((fn=getenv("SDL_RECORD_PATH"))==NULL) {
	fn=default_filename;
    }
    if((out=open(fn,O_WRONLY|O_CREAT,0755))==-1) {
	strerror_r(errno,err_buff,sizeof(err_buff));
	fprintf(stderr,"Can't open file %s: %s\n",fn,err_buff);
	exit(0);
    }
    
    header.magic=MAGIC;
    header.version=FILEVERSION;
    gettimeofday(&header.starttime,NULL);
    if(write(out,&header,sizeof(header))!=sizeof(header)) {
	strerror_r(errno,err_buff,sizeof(err_buff));
	fprintf(stderr,"Can't write to file %s: %s\n",fn,err_buff);
	exit(0);
    }
    return(out);
}

int call_sdl_func(int (*f)(),SDL_Event *e) {
    static int out=0; /* output file fd */
    static int lastflush=0; 
    int r;
    struct timeval tp;
    char err_buff[128]; /* for strerror_r (which we want to use in case this */
                        /* is threaded) */

    if(!out) {
	out=open_file();
    }
    if(!(r=f(e))) {
	return(0);
    }

    gettimeofday(&tp,NULL);
    if((write(out,&tp,sizeof(tp))!=sizeof(tp))||
       (write(out,e,sizeof(*e))!=sizeof(*e))) {
	strerror_r(errno,err_buff,sizeof(err_buff));
	fprintf(stderr,"Can't write to output files: %s\n",err_buff);
	exit(0);
    }
    /* we ensure that we fsync every 5 secs, since your app may crash or who 
     * knows what and not get synced */
    if(tp.tv_sec-lastflush>5) {
	fsync(out);
	lastflush=tp.tv_sec;
    }
    return(r);
}


